@@ -1,38 +1,23 @@
-##   The contents of this file are subject to the Mozilla Public License
-##   Version 1.1 (the "License"); you may not use this file except in
-##   compliance with the License. You may obtain a copy of the License at
-##   http://www.mozilla.org/MPL/
+##  The contents of this file are subject to the Mozilla Public License
+##  Version 1.1 (the "License"); you may not use this file except in
+##  compliance with the License. You may obtain a copy of the License
+##  at http://www.mozilla.org/MPL/
 ##
-##   Software distributed under the License is distributed on an "AS IS"
-##   basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
-##   License for the specific language governing rights and limitations
-##   under the License.
+##  Software distributed under the License is distributed on an "AS IS"
+##  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
+##  the License for the specific language governing rights and
+##  limitations under the License.
 ##
-##   The Original Code is RabbitMQ.
+##  The Original Code is RabbitMQ.
 ##
-##   The Initial Developers of the Original Code are LShift Ltd,
-##   Cohesive Financial Technologies LLC, and Rabbit Technologies Ltd.
-##
-##   Portions created before 22-Nov-2008 00:00:00 GMT by LShift Ltd,
-##   Cohesive Financial Technologies LLC, or Rabbit Technologies Ltd
-##   are Copyright (C) 2007-2008 LShift Ltd, Cohesive Financial
-##   Technologies LLC, and Rabbit Technologies Ltd.
-##
-##   Portions created by LShift Ltd are Copyright (C) 2007-2009 LShift
-##   Ltd. Portions created by Cohesive Financial Technologies LLC are
-##   Copyright (C) 2007-2009 Cohesive Financial Technologies
-##   LLC. Portions created by Rabbit Technologies Ltd are Copyright
-##   (C) 2007-2009 Rabbit Technologies Ltd.
-##
-##   All Rights Reserved.
-##
-##   Contributor(s): ______________________________________.
+##  The Initial Developer of the Original Code is VMware, Inc.
+##  Copyright (c) 2007-2011 VMware, Inc.  All rights reserved.
 ##
 
 from __future__ import nested_scopes
 import re
 import sys
-from os import remove
+import os
 from optparse import OptionParser
 
 try:
@@ -172,7 +157,7 @@ class AmqpSpec:
 
         self.classes = []
         for element in self.spec['classes']:
-            self.classes.append(AmqpClass(self.spec, element))
+            self.classes.append(AmqpClass(self, element))
         
     def allClasses(self):
         return self.classes
@@ -271,16 +256,16 @@ def do_main_dict(funcDict):
     def execute(fn, amqp_specs, out_file):
         stdout = sys.stdout
         f = open(out_file, 'w')
+        success = False
         try:
-            try:
-                sys.stdout = f
-                fn(amqp_specs)
-            except:
-                remove(out_file)
-                raise
+            sys.stdout = f
+            fn(amqp_specs)
+            success = True
         finally:
             sys.stdout = stdout
             f.close()
+            if not success:
+                os.remove(out_file)
 
     parser = OptionParser()
     parser.add_option("--ignore-conflicts", action="store_true", dest="ignore_conflicts", default=False)
